package Grenouille::System;

# This software may be used and distributed according to the terms
# of the GNU General Public License, incorporated herein by reference.

use strict;
use Exporter qw();
use Socket;
use Net::SNMP;
use Grenouille::Vars qw($USER);
use Grenouille::Preferences qw(%preferences);
use Grenouille::Log qw(display_and_log);
use vars qw(@ISA @EXPORT $session);
@ISA=qw(Exporter);
@EXPORT=qw(get_traffic_counters autodetect_interface
           init_privileges swap_privileges);

my $MIB ||= '1.3.6.1.2.1';
my $interfaces ||= "$MIB.2";
my $ip ||= "$MIB.4";

# internet 1.3.6.1
# mgmt 2
# mib 1
# 
# system 1
# interfaces 2
#         ifNumber 1.0
#         ifTable 2
#                 ifEntry 1
#                         (ifIndex                 1.x
#                          ifDescr                 2.x
#                          ifType                  3.x
#                          ifMtu                   4.x
#                          ifSpeed                 5.x
#                          ifPhysAddress           6.x
#                          ifAdminStatus           7.x
#                          ifOperStatus            8.x
#                          ifLastChange            9.x
#                          ifInOctets             10.x
#                          ifInUcastPkts          11.x
#                          ifInNUcastPkts         12.x
#                          ifInDiscards           13.x
#                          ifInErrors             14.x
#                          ifInUnknownProtos      15.x
#                          ifOutOctets            16.x
#                          ifOutUcastPkts         17.x
#                          ifOutNUcastPkts        18.x
#                          ifOutDiscards          19.x
#                          ifOutErrors            20.x
#                          ifOutQLen              21.x)
# ip 4
#         ipAddrTable 20
#                 ipAddrEntry 1
#                         (ipAdEntAddr             1.x
#                          ipAdEntIfIndex          2.x
#                          ipAdEntNetMask          3.x
#                          ipAdEntBcastAddr        4.x)
#         ipRoutingTable 21
#                 ipRouteEntry 1
#                         (ipRouteDest             1.x
#                          ipRouteIfIndex          2.x
#                          ipRouteMetric1          3.x
#                          ipRouteMetric2          4.x
#                          ipRouteMetric3          5.x
#                          ipRouteMetric4          6.x
#                          ipRouteNextHop          7.x
#                          ipRouteType             8.x
#                          ipRouteProto            9.x
#                          ipRouteAge             10.x)

$session = undef;

sub _open_snmp_session() {
    return if defined $session;

    my($error, $port);
    $port = getservbyname('snmp', 'udp');
    ($session, $error) = 
      Net::SNMP->session(
                         -hostname  => $preferences{'snmp_host'},
                         -community => $preferences{'snmp_community'},
                         -port      => $port
                         );
    if(not defined $session) {
        display_and_log('ERROR', "snmp: $error");
        die "Could not open SNMP session host=$preferences{'snmp_host'} port=$port community=$preferences{'snmp_community'}: $error";
    }
}

sub _close_snmp_session() {
    return unless defined $session;
    display_and_log('DEBUG', 'Close SNMP session');
    $session->close();
    undef $session;
}

# returns rx and tx byte counters of the given interface
sub get_traffic_counters($) {
    my ($if_device) = @_;
    my($result, $k, $no);

    _open_snmp_session();
                             # interfaces.ifTable.ifEntry.ifDescr
    $session->get_table("$interfaces.2.1.2") or die $session->error();
    $result = $session->var_bind_list;
    $no = undef;
    foreach $k (keys %$result) {
        if($result->{$k} eq $if_device) {
            $k =~ m/\.(\d+)$/;
            $no = $1;
            last;
        }
    }
    die "Could not find interface $if_device" unless defined $no;
    my(@oids) = ("$interfaces.2.1.10.$no", "$interfaces.2.1.16.$no");
    $session->get_request(@oids) or die $session->error();
    $result = $session->var_bind_list;
    return { 'rx' => $result->{$oids[0]}, 'tx' => $result->{$oids[1]} };
}

sub autodetect_interface() {
    my($result, $k, $no, $if, $if_ip);

    eval {
        _open_snmp_session();
                    # ip.ipRoutingTable.ipRouteEntry.ipRouteDest
        $session->get_table("$ip.21.1.1") or die $session->error();
        $result = $session->var_bind_list;
        $no = undef;
        foreach $k (keys %$result) {
            if($result->{$k} eq '0.0.0.0') {
                $no = $k;
                $no =~ s/^$ip.21.1.1.//;
                last;
            }
        }
        return undef unless defined $no;

             # ip.ipRoutingTable.ipRouteEntry.ipRouteIfIndex
        my($oid) = "$ip.21.1.2.$no";
        $session->get_request($oid) or die $session->error();
        $result = $session->var_bind_list;
        $no = $result->{$oid};
             # interfaces.ifTable.ifEntry.ifDescr
        $oid = "$interfaces.2.1.2.$no";
        $session->get_request($oid) or die $session->error();
        $result = $session->var_bind_list;
        $if = $result->{$oid};
	     # ip.ipAddrTable.ipAddrEntry.ipAdEntIfIndex
	$session->get_table("$ip.20.1.2") or die $session->error();
	$result = $session->var_bind_list;
	$if_ip = undef;
	foreach my $k (keys %$result) {
	    if($result->{$k} == $no) {
		$k =~ m/\.(\d+\.\d+\.\d+\.\d+)$/;
		$if_ip = $1;
		last;
	    }
	}
    };
    if ($@) {
        display_and_log('ERROR', "snmp: $@");
        return (undef, undef);
    }
    return ($if, $if_ip);
}

END {
    _close_snmp_session();
}

sub swap_privileges() {
    if($^O ne 'win32') {
        ($<, $>) = ($>, $<);
        ($(, $)) = ($), $();
    }
}

sub init_privileges() {
    if($^O ne 'win32') {
        my(undef, undef, $uid, $gid) = getpwnam($USER) or 
            die "$USER absent de /etc/password";
        
        $< = $uid;
        $( = $gid;
        
        die "Dsol, ce programme doit avoir les droits de root pour pouvoir envoyer des pings ICMP" if $>;
 
        swap_privileges();
    }
}

1;
